/*
 * Creado por SharpDevelop.
 * Usuario: Daniel Doctor
 * Visita www.ubicuos.com
 * Fecha: 09/04/2008
 * Hora: 11:48 a.m.
 * 
 * Adaptado por Mauro Lopes
 * 
 * 
 */

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace DDA
{
    /// <summary>
    /// Description of MainForm.
    /// </summary>
    public partial class frmDDA : Form
    {
        // define os atributos
        public Color color;
        private Graphics g;
        private Bitmap bm;

        public frmDDA()
        {
            //
            // The InitializeComponent() call is required for Windows Forms designer support.
            //
            InitializeComponent();

            //
            // TODO: Add constructor code after the InitializeComponent() call.
            //
        }

        /**
         * Desenha um pixel na posio (x,y) na cor c
         * 
         */
        void desenharPixel(int x, int y, Color c)
        {
            bm.SetPixel(x, y, c);
        }

        /*
         * Desenha uma linha usando o algoritmo de Breseham
         * entre o ponto inicial (xa,ya) at o ponto final (xb,yb)
         * 
         */
        void linhaBresenham(int xa, int ya, int xb, int yb)
        {
            int dx, dy, x, y, xEnd, p;
            int xIncrement, yIncrement;
            int aux;

            bool mMaior = false;

            dx = Math.Abs(xa - xb);
            dy = Math.Abs(ya - yb);

            // Tratamos casos especiais dx=0,  dy=0 , dx=dy
            if (dx == dy || dx == 0 || dy == 0)
            {

                int k, steps;

                steps = Math.Max(dx, dy);

                if (xb > xa)
                {
                    xIncrement = 1;
                }
                else
                {
                    xIncrement = -1;
                }

                if (yb > ya)
                {
                    yIncrement = 1;
                }
                else
                {
                    yIncrement = -1;
                }

                if (dx == 0)
                {
                    xIncrement = 0;
                }

                if (dy == 0)
                {
                    yIncrement = 0;
                }

                x = xa;
                y = ya;

                for (k = 1; k <= steps; k++)
                {

                    x += xIncrement;
                    y += yIncrement;

                    desenharPixel(x, y, color);
                }

            }
            else
            {

                if (Math.Abs(dy) > Math.Abs(dx))
                { 
                    //intercambio x e y para pendientes mayores que 1
                    mMaior = true;
                    aux = xa;
                    xa = ya;
                    ya = aux;
                    aux = xb;
                    xb = yb;
                    yb = aux;
                    dx = Math.Abs(xa - xb);
                    dy = Math.Abs(ya - yb);
                }

                p = 2 * dy - dx;
                // determinar o ponto de inicio

                if (xa > xb)
                {
                    // linhas com y decrescente
                    if (ya < yb)
                    {
                        yIncrement = -1;
                    }
                    else
                    {
                        yIncrement = 1;
                    }

                    x = xb;
                    y = yb;

                    xEnd = xa;

                }
                else
                {
                    if (ya > yb)
                    {
                        yIncrement = -1;
                    }
                    else
                    {
                        yIncrement = 1;
                    }

                    x = xa;
                    y = ya;

                    xEnd = xb;

                }

                if (mMaior)
                {
                    desenharPixel(y, x, color);
                }
                else
                {
                    desenharPixel(x, y, color);
                }

                while (x < xEnd)
                {
                    x++;

                    if (p < 0)
                    {
                        p += 2 * dy;
                    }
                    else
                    {
                        y += yIncrement;
                        p += 2 * (dy - dx);
                    }
                    if (mMaior)
                    {
                        desenharPixel(y, x, color);
                    }
                    else
                    {
                        desenharPixel(x, y, color);
                    }
                }

            }

        }

        void linhaDDA(int x0,int y0, int x1, int y1)
        {
            int dx = (int)Math.Abs(x1 - x0);
            int dy = (int)Math.Abs(y1 - y0);
           
            int s;

            int x = x0;
            int y = y0;

            //Console.WriteLine("dx={0}, dy={1}",dx,dy);

            if ((dx) > (dy))
            {
                // sim
                s = dx;
            }
            else
            {
                // no
                s = dy;
            }

            //Console.WriteLine("s={0}", s);

            int xi = dx / s;
            int yi = dy / s;

            //Console.WriteLine("xi={0},yi={1}", xi, yi);

            //Console.WriteLine("x={0},y={1}\n", x, y);

            //printf("x=%f,y=%f\n",x,y);
            //pinta(round(x), round(y));
            desenharPixel(x, y, color);

            int k = 0;

            while (k < s)
            {
                x = x + xi;
                y = y + yi;

                //printf("x=%f,y=%f\n",x,y);
                //pinta(round(x), round(y));
                //Console.WriteLine("x={0},y={1}", x, y);
                desenharPixel(x, y, color);

                k++;

            }   
        }

        void MainFormLoad(object sender, EventArgs e)
        {
            // cria um contexto grfico
            g = this.CreateGraphics();

            // Cria um novo bitmap com base no tamanho do formulario
            bm = new Bitmap(this.Width, this.Height);

            // Estabelece a cor para o desenho da linha
            color = Color.Red;

        }
    }
}
